//--------------------------------------------------------------
// Fichero:  cronart.js
// Objetivo: Rutinas JavaScript de CronArt
// Fecha:    J.9.7.2015
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//---------------------------------
// Constantes
//---------------------------------

// Cuánto modificar el tamaño de la zona de dibujo
var PasoCambioTamano = 10;

//---------------------------------
// Variables globales
//---------------------------------

// Los elementos HTML
var BotonRedibujar;
var BotonEmpezar;
var BotonParar;
var BotonCompleta;
var BotonAncho;
var BotonEstrecho;
var BotonAlto;
var BotonBajo;
var SelectTamano;
var CanvasDibujo;
var Contexto;
var BotonHistoria;
var BotonCreditos;
var BotonLicencia;
var BotonDescarga;
var BotonTecnicas;
var BotonCerrar1;
var BotonCerrar2;
var BotonCerrar3;
var BotonCerrar4;
var BotonCerrar5;
var DivHistoria;
var DivCreditos;
var DivLicencia;
var DivDescarga;
var DivTecnicas;

// Valores mínimo y máximo de filas y columnas
var MaxFil;
var MinFil;
var MaxCol;
var MinCol;

// Centro de la zona de dibujo
var CentroX;
var CentroY;

// El tamaño de los dígitos
var TamanoTexto;

// El tamaño de cada celda
var TamanoCelda;

// Los arrays de contadores
var Contador = [];
var Lapso = [];
var Transcurrido = [];
var TotalContador = 0;

// El manejador del temporizador
var Temporizador;

// Para controlar el tiempo
var Antes;
var Ahora;

// Tamaño de la zona de dibujo antes de pasar a pantalla completa
var AntiguaAnchura
var AntiguaAltura

//---------------------------------
// Funciones
//---------------------------------

//--------------------------------------------------------------
// Reiniciar el sistema de dibujo
function Reiniciar()
  {
  // Obtenemos el contexto de la zona para dibujar
  Contexto = CanvasDibujo.getContext("2d");

  // Dimensiones de la zona para dibujar
  var Anchura = Contexto.canvas.clientWidth;
  var Altura = Contexto.canvas.clientHeight;

  // Borramos la zona para dibujar
  Contexto.fillStyle = "#000000";
  Contexto.fillRect (0, 0, Anchura, Altura);

  // Establecemos las características de la letra
  Contexto.fillStyle = "#FF0000";
  Contexto.font = TamanoTexto.toString() + "px digito";
  Contexto.textAlign = "center";
  Contexto.textBaseline = "middle";

  // Dividimos la zona de dibujo en celdas cuadradas
  TamanoCelda = Contexto.measureText("00").width;

  // Calculamos el punto central de la zona de dibujo
  CentroX = Anchura/2;
  CentroY = Altura/2;

  // Calculamos las filas y columnas que caben en la zona
  // de modo que la celda (0,0) quede en el centro
  MaxFil = Math.floor((((Altura - TamanoCelda) / 2) / TamanoCelda) / 2);
  MinFil = -MaxFil;
  MaxCol = Math.floor((((Anchura - TamanoCelda) / 2) / TamanoCelda) / 2);
  MinCol = -MaxCol;

  // Calculamos cuántos contadores necesitamos
  TotalContador = (MaxFil-MinFil+1) * (MaxCol-MinCol+1);

  // Vaciamos los arrays de contadores, lapsos y tiempos transcurridos
  Contador = [];
  Lapso = [];
  Transcurrido = [];

  // Damos valores iniciales a los tres arrays
  for ( var i=0 ; i<TotalContador ; i++ )
    {
    // Iniciamos los contadores con números de 0 a 99
    Contador.push (Aleatorio(99));

    // Iniciamos los lapsos con números aleatorios entre 100 y 2000
    Tiempo = 100 + 100 * Aleatorio(19);
    Lapso.push (Tiempo);

    // Rellenamos al azar el tiempo transcurrido
    Transcurrido.push (Aleatorio(Tiempo));
    }

  // Recorremos todas las celdas con números y los escribimos
  var i=0;
  for ( var Fil=MinFil ; Fil<=MaxFil ; Fil++ )
    {
    for ( var Col=MinCol ; Col<=MaxCol ; Col++ , i++ )
      { EscribeContador (i, Col, Fil); }
    }
  }

//--------------------------------------------------------------
// Escribe un contador en una celda determinada
function EscribeContador (i, Col, Fil)
  {
  var Num = Contador[i];
  var Texto = ("0" + Num).slice(-2);
  Contexto.fillText (Texto, CentroX + 2*TamanoCelda*Col,
                            CentroY + 2*TamanoCelda*Fil);
  }

//--------------------------------------------------------------
// Empezar a cambiar los valores de los contadores
function Empezar()
  {
  // Cambiamos la activación de los botones
  BotonParar.disabled = false;
  BotonEmpezar.disabled = true;

  // Preparamos los tiempos
  Antes = 0;
  Ahora = Reloj();

  // Activamos el temporizador cada 0.1 s
  Temporizador = setInterval (Cambiar, 100);
  }

//--------------------------------------------------------------
// Parar de cambiar los valores de los contadores
function Parar()
  {
  // Cambiamos la activación de los botones
  BotonParar.disabled = true;
  BotonEmpezar.disabled = false;

  // Eliminamos el temporizador
  window.clearInterval (Temporizador);
  }

//--------------------------------------------------------------
// Cambiar los valores de los contadores
function Cambiar()
  {
  // Guardamos el tiempo anterior
  Antes = Ahora;

  // Tomamos nota del tiempo actual
  Ahora = Reloj();

  // Vemos cuánto tiempo ha pasado
  TiempoEntreLlamadas = Ahora - Antes;

  // Recorremos todas las celdas con números
  var i=0;
  for ( var Fil=MinFil ; Fil<=MaxFil ; Fil++ )
    {
    for ( var Col=MinCol ; Col<=MaxCol ; Col++ , i++ )
      {
      // Aumentamos el tiempo transcurrido
      Transcurrido[i] = Transcurrido[i]+TiempoEntreLlamadas;

      // Comprobamos si se ha alcanzado el Lapso
      if ( Transcurrido[i] >= Lapso[i] )
        {
        // Ponemos el tiempo transcurrido en cero
        Transcurrido[i] = 0;

        // Borramos el número anterior con un pequeño cuadrado negro
        Contexto.fillStyle = "#000000";
        Contexto.fillRect (CentroX + 2*(TamanoCelda*(Col-0.25)),
                           CentroY + 2*(TamanoCelda*(Fil-0.25)),
                           TamanoCelda, TamanoCelda);

        // Aumentamos el contador
        Contador[i]++;
        if ( Contador[i] == 100 )
          { Contador[i] = 0; }

        // Y lo escribimos
        Contexto.fillStyle = "#FF0000";
        EscribeContador (i, Col, Fil);
        }
      }
    }
  }

//--------------------------------------------------------------
// Devuelve el número de milisegundos desde 1970-01-01
function Reloj()
  { return (new Date()).getTime(); }

//--------------------------------------------------------------
// Devuelve un número entero aleatorio entre 0 y el dado
function Aleatorio (Max)
  { return Math.floor(Math.random()*Max); }

//--------------------------------------------------------------
// Si nos piden pasar a pantalla completa
function PantallaCompleta()
  {
  // Guardamos las dimensiones de la zona de dibujo
  AntiguaAnchura = Contexto.canvas.clientWidth;
  AntiguaAltura = Contexto.canvas.clientHeight;

  // Llevamos a pantalla completa el área de dibujo
  ElementoPantallaCompleta (CanvasDibujo);

  // Cambiamos el tamaño del canvas al máximo posible
  CanvasDibujo.width = window.screen.availWidth;
  CanvasDibujo.height = window.screen.availHeight;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Pasar a pantalla completa un elemento del DOM
function ElementoPantallaCompleta (Elemento)
  {
  // Método genérico
  if (Elemento.requestFullscreen)
    { Elemento.requestFullscreen(); }

  // Webkit
  else if (Elemento.webkitRequestFullscreen)
    { Elemento.webkitRequestFullscreen(); }

  // Mozilla
  else if (Elemento.mozRequestFullScreen)
    { Elemento.mozRequestFullScreen(); }

  // Otro navegador
  else if (Elemento.msRequestFullscreen)
    { Elemento.msRequestFullscreen(); }
  }

//--------------------------------------------------------------
// Dice si estamos en modo pantalla completa
function EstamosPantallaCompleta()
  {
  Respuesta = false;

  if ( document.fullscreenElement    || document.webkitFullscreenElement ||
       document.mozFullScreenElement || document.msFullscreenElement )
    { Respuesta = true; }

  return Respuesta;
  }

//--------------------------------------------------------------
// Si nos cambian el modo de pantalla completa
function CambioEnPantallaCompleta()
  {
  if ( ! EstamosPantallaCompleta() )
    {
    // Dejamos todo como estaba
    CanvasDibujo.width = AntiguaAnchura;
    CanvasDibujo.height = AntiguaAltura;
    Reiniciar();
    }
  }

//--------------------------------------------------------------
// Si nos cambian el tamaño del texto
function CambiaTamano()
  {
  // Anotamos el nuevo tamaño
  TamanoTexto = SelectTamano.value;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Si nos piden hacer más ancha la zona de dibujo
function MasAncho()
  {
  // Cambiamos el tamaño
  CanvasDibujo.width = Contexto.canvas.clientWidth + PasoCambioTamano;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Si nos piden hacer más estrecha la zona de dibujo
function MasEstrecho()
  {
  // Cambiamos el tamaño
  CanvasDibujo.width = Contexto.canvas.clientWidth - PasoCambioTamano;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Si nos piden hacer más alta la zona de dibujo
function MasAlto()
  {
  // Cambiamos el tamaño
  CanvasDibujo.height = Contexto.canvas.clientHeight + PasoCambioTamano;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Si nos piden hacer más baja la zona de dibujo
function MasBajo()
  {
  // Cambiamos el tamaño
  CanvasDibujo.height = Contexto.canvas.clientHeight - PasoCambioTamano;

  // Volvemos a dibujar
  Reiniciar();
  }

//--------------------------------------------------------------
// Activa la visión del DIV de Historia
function Historia()
  {
  Cierra();
  DivHistoria.style.display = "block";
  }

//--------------------------------------------------------------
// Activa la visión del DIV de Créditos
function Creditos()
  {
  Cierra();
  DivCreditos.style.display = "block";
  }

//--------------------------------------------------------------
// Activa la visión del DIV de Licencia
function Licencia()
  {
  Cierra();
  DivLicencia.style.display = "block";
  }

//--------------------------------------------------------------
// Activa la visión del DIV de Descarga
function Descarga()
  {
  Cierra();
  DivDescarga.style.display = "block";
  }

//--------------------------------------------------------------
// Activa la visión del DIV de Técnicas
function Tecnicas()
  {
  Cierra();
  DivTecnicas.style.display = "block";
  }

//--------------------------------------------------------------
// Cierra todos los DIV de explicaciones
function Cierra()
  {
  DivHistoria.style.display = "none";
  DivCreditos.style.display = "none";
  DivLicencia.style.display = "none";
  DivDescarga.style.display = "none";
  DivTecnicas.style.display = "none";
  }

//--------------------------------------------------------------
// Función portable que añade un evento
function AgregaEvento (elemento, evType, funcion, useCapture)
  {
  // Manejador de eventos compatible con navegadores IE5+, NS6 y Mozilla
  // Autor: Scott Andrew
  if ( elemento.addEventListener )
    {
    elemento.addEventListener (evType, funcion, useCapture);
    return true;
    }
  else if ( elemento.attachEvent )
    {
    var r = elemento.attachEvent ('on' + evType, funcion);
    return r;
    }
  else
    { elemento ['on' + evType] = funcion; }
  }

//--------------------------------------------------------------
// La función que inicia todo el sistema
function IniciaTodo()
  {
  // Averiguamos los ID de los elementos
  BotonReiniciar = document.getElementById ('reiniciar');
  BotonEmpezar = document.getElementById ('empezar');
  BotonParar = document.getElementById ('parar');
  BotonCompleta = document.getElementById ('completa');
  BotonAncho = document.getElementById ('ancho');
  BotonEstrecho = document.getElementById ('estrecho');
  BotonAlto = document.getElementById ('alto');
  BotonBajo = document.getElementById ('bajo');
  BotonHistoria = document.getElementById ('historia');
  BotonCreditos = document.getElementById ('creditos');
  BotonLicencia = document.getElementById ('licencia');
  BotonDescarga = document.getElementById ('descarga');
  BotonTecnicas = document.getElementById ('tecnicas');
  BotonCerrar1 = document.getElementById ('botoncerrar1');
  BotonCerrar2 = document.getElementById ('botoncerrar2');
  BotonCerrar3 = document.getElementById ('botoncerrar3');
  BotonCerrar4 = document.getElementById ('botoncerrar4');
  BotonCerrar5 = document.getElementById ('botoncerrar5');
  CanvasDibujo = document.getElementById ('dibujo');
  SelectTamano = document.getElementById ('tamano');
  DivHistoria = document.getElementById ('divhistoria');
  DivCreditos = document.getElementById ('divcreditos');
  DivLicencia = document.getElementById ('divlicencia');
  DivDescarga = document.getElementById ('divdescarga');
  DivTecnicas = document.getElementById ('divtecnicas');

  // Valores iniciales
  TamanoTexto = 16;
  CanvasDibujo.width = window.innerWidth - 60;

  // Añadimos todos los "listeners" necesarios
  AgregaEvento (BotonReiniciar, 'click', Reiniciar, false);
  AgregaEvento (BotonEmpezar, 'click', Empezar, false);
  AgregaEvento (BotonParar, 'click', Parar, false);
  AgregaEvento (BotonCompleta, 'click', PantallaCompleta, false);
  AgregaEvento (BotonAncho, 'click', MasAncho, false);
  AgregaEvento (BotonEstrecho, 'click', MasEstrecho, false);
  AgregaEvento (BotonAlto, 'click', MasAlto, false);
  AgregaEvento (BotonBajo, 'click', MasBajo, false);
  AgregaEvento (BotonHistoria, 'click', Historia, false);
  AgregaEvento (BotonCreditos, 'click', Creditos, false);
  AgregaEvento (BotonLicencia, 'click', Licencia, false);
  AgregaEvento (BotonDescarga, 'click', Descarga, false);
  AgregaEvento (BotonTecnicas, 'click', Tecnicas, false);
  AgregaEvento (BotonCerrar1, 'click', Cierra, false);
  AgregaEvento (BotonCerrar2, 'click', Cierra, false);
  AgregaEvento (BotonCerrar3, 'click', Cierra, false);
  AgregaEvento (BotonCerrar4, 'click', Cierra, false);
  AgregaEvento (BotonCerrar5, 'click', Cierra, false);
  AgregaEvento (SelectTamano, 'change', CambiaTamano, false);

  // El manejador de cambio de pantalla completa
  document.addEventListener ("fullscreenchange", CambioEnPantallaCompleta);
  document.addEventListener ("webkitfullscreenchange", CambioEnPantallaCompleta);
  document.addEventListener ("mozfullscreenchange", CambioEnPantallaCompleta);
  document.addEventListener ("MSFullscreenChange", CambioEnPantallaCompleta);

  // Arrancamos
  Reiniciar();
  }

//--------------------------------------------------------------
// La función que inicia el sistema cuando se carga la página
AgregaEvento (window, 'load', IniciaTodo, false);
